# -*- coding: utf-8 -*-
import datetime
from decimal import Decimal

import django.contrib.auth.models as auth_models
import django.db as db
import django.db.models as models
import django.forms as forms
from django.conf import settings
from django.contrib import admin

import basket

import gaby.contacts.fields_choices as fch
import gaby.references.models as ref
from gaby.contacts.logger import logger
from gaby.references.fields_choices import AGENCES_EAU


def lin_choice(nested_ch):
    res = []
    for elm in nested_ch:
        res += elm[1]
    return res


# ## Classes utilitaires
class AsDlMixin(object):
    def as_dl(self):
        "Returns this form rendered as HTML <dt>s -- excluding the <dl></dl>."
        return self._html_output(
            normal_row='<dt%(html_class_attr)s>%(label)s</dt>'
            '<dd>%(field)s %(help_text)s </dd>',
            error_row='<dt>%s</dt>',
            row_ender='</dt>',
            help_text_html=' <div class="helptext">%s</div>',
            errors_on_separate_row=True,
        )


# ## Données et fonctions utilitaires
def coerce_radionullbool(val, inverse=False):
    """
    Fonction accessoire utilisée par les TypedChoiceField affichant un boolean
    pouvant être nul, sous forme de boutons radio (plus ergonomique que le
    selection utilisé par défaut par NullBooleanField).
    Si inverse=True, on retourne True pour oui, pour pouvoir utiliser le
    booleen avec une requête SQL qui utilise __isnull (qui doit être True
    lorsqu'il n'y a *pas* d'objet en relation).
    """
    if val == 'oui':
        return not inverse
    if val == 'non':
        return inverse
    return None


def coerce_radionullbool_inverse(val):
    """
    Similaire à coerce_radionullbool, mais renvoit False pour oui et True pour
    non pour être utilisé directement dans une requête __isnull qui doit être
    True quand il n'y a pas d'objet en relation (ex: pas d'adhésion pour
    un agriculteur bio).
    """
    return coerce_radionullbool(val, inverse=True)


# ## Form de filtres spéciaux des listes
class FormFiltre(AsDlMixin, forms.Form):
    """
    Form de base pour filtrer une liste d'objets dans l'admin dango selon des
    filtres avancés (contrairement aux filtres fournis par django, on peut
    sélectionner plusieurs valeurs, un 'OU' étant appliquées entre ces
    valeurs).
    """

    class Media:
        css = {
            # forms.css nécessaire pour le style des filtres spéciaux
            "all": (
                settings.STATIC_URL + "admin/css/forms.css",
                settings.STATIC_URL + "css/contacts_forms.css",
            )
        }
        js = (settings.STATIC_URL + "admin/js/collapse.min.js",)

    # correspondance entre champs et argument de filtre du queryset
    filtres_qs = {}

    def construit_filtres_qs(self, cleaned_data):
        """
        Retourne une liste d'objets Q pour filtrer un queryset à partir des
        données du formulaire (qui sont conservées en session, c'est pourquoi
        on les passe en argument).
        (cette méthode pourrait aussi bien figurer dans la vue -- dans
        contacts_admin.MixinAdminFiltrable --, mais elle dépend aussi fortement
        des données du formulaire filtres_qs, on peut donc la considérer comme
        liée au formulaire.)
        """
        q_list = []
        for f in list(self.filtres_qs.keys()):
            f_val = cleaned_data.get(f, [])
            # logger.debug("  valeur %s : %s" % (f, f_val))
            if isinstance(f_val, models.query.QuerySet):
                q_val = [o.id for o in f_val]
            elif isinstance(f_val, list):
                if len(f_val) == 0:
                    # un QuerySet vide apparaît comme une liste
                    continue
                q_val = f_val
            elif (
                isinstance(f_val, bool)
                or isinstance(f_val, Decimal)
                or isinstance(f_val, datetime.date)
            ):
                q_val = f_val
            else:
                # S'il s'agit d'autre chose, c'est un filtre vide
                # qu'on ignore (une date non choisie vaut None, etc.)
                continue
            logger.debug("+ filtre: %s" % {self.filtres_qs[f]: q_val})
            q_list.append(models.Q(**{self.filtres_qs[f]: q_val}))
        return q_list


class FormFiltreStructure(basket.forms.FormBigFilter):
    # class FormFiltreStructure(FormFiltre):
    """
    Formulaire pour filtrer la liste des structures.
    """
    filtered_qs = dict(
        statutbio="statutbio__in",
        etiquettes="etiquettes__id__in",
        statut="statut__id__in",
        territoire="commune__territoire__id__in",
        commune_agence_eau="commune__agence_eau__in",
        commune_enjeu_eau="commune__enjeu_eau",
        type_rendez_vous="rdv_ind_set__type_rdv__in",
        technicien_suivi="projetbio__technicien__id__in",
        technicien_rdv="projetbio__rdv__technicien__id__in",
        date_rdv_anterieure="projetbio__rdv__date__gte",
        date_rdv_posterieure="projetbio__rdv__date__lte",
        activitestructure_nom='activitestructure__nom__in',
        activitestructure_bio='activitestructure__activite_bio__in',
        adherent="adhesion__isnull",
        montant_adhesion_min="adhesion__montant__gte",
        montant_adhesion_max="adhesion__montant__lte",
        date_adhesion_anterieure="premiere_adhesion__date__gte",
        date_adhesion_posterieure="premiere_adhesion__date__lte",
        date_renouvellement_anterieure="renouvellement_adhesion__date__gte",
        date_renouvellement_posterieure="renouvellement_adhesion__date__lte",
    )
    filtre_qs = filtered_qs

    statutbio = forms.MultipleChoiceField(
        required=False,
        widget=forms.CheckboxSelectMultiple,
        label="Statut bio",
        choices=fch.STATUT_BIO,
    )

    etiquettes = forms.ModelMultipleChoiceField(
        required=False,
        queryset=ref.Etiquette.objects.all(),
        widget=forms.CheckboxSelectMultiple,
        label="Étiquettes",
    )

    statut = forms.ModelMultipleChoiceField(
        required=False,
        queryset=ref.StatutStructure.objects.all(),
        widget=forms.CheckboxSelectMultiple,
        label="Statut",
    )

    territoire = forms.ModelMultipleChoiceField(
        required=False,
        queryset=ref.Territoire.objects.all(),
        widget=forms.CheckboxSelectMultiple,
        label="Territoire",
    )

    commune_agence_eau = forms.MultipleChoiceField(
        required=False,
        widget=forms.CheckboxSelectMultiple,
        label="Agence eau ?",
        choices=AGENCES_EAU,
    )

    commune_enjeu_eau = forms.TypedChoiceField(
        required=False,
        choices=fch.CHOIX_NULLBOOLEAN,
        initial='ind',
        coerce=coerce_radionullbool,
        widget=forms.RadioSelect,
        label="Sur commune à enjeu eau ?",
    )

    activitestructure_nom = forms.MultipleChoiceField(
        required=False,
        choices=fch.ACTIVITE_LISTE,
        widget=forms.CheckboxSelectMultiple,
        label="Activité (nom)",
    )

    activitestructure_bio = forms.MultipleChoiceField(
        required=False,
        choices=fch.ACTIVITE_BIO,
        widget=forms.CheckboxSelectMultiple,
        label="Activité (bio?)",
    )

    technicien_suivi = forms.ModelMultipleChoiceField(
        required=False,
        widget=forms.CheckboxSelectMultiple,
        queryset=(
            auth_models.User.objects.exclude(username='admin')
            .annotate(group_count=db.models.Count('groups'))
            .filter(is_active=True, group_count__gte=1)
            .order_by('username')
        ),
        label="Ayant au moins un projet bio suivi par",
    )

    type_rendez_vous = forms.MultipleChoiceField(
        required=False,
        widget=forms.CheckboxSelectMultiple,
        choices=lin_choice(fch.CHOIX_TYPE_RDV),
        label="Ayant au moins un rendez-vous pour projet bio de type",
    )

    technicien_rdv = forms.ModelMultipleChoiceField(
        required=False,
        widget=forms.CheckboxSelectMultiple,
        queryset=auth_models.User.objects.exclude(username='admin')
        .annotate(group_count=db.models.Count('groups'))
        .filter(is_active=True, group_count__gte=1)
        .order_by('username'),
        label="Ayant au moins un rendez-vous pour projet bio effectué par",
    )

    date_rdv_anterieure = forms.DateField(
        required=False,
        widget=admin.widgets.AdminDateWidget,
        label="Ayant au moins un rendez-vous pour projet bio après le",
    )

    date_rdv_posterieure = forms.DateField(
        required=False,
        widget=admin.widgets.AdminDateWidget,
        label="Ayant au moins un rendez-vous pour projet bio avant le",
    )

    adherent = forms.TypedChoiceField(
        required=False,
        choices=fch.CHOIX_NULLBOOLEAN,
        initial='ind',
        coerce=coerce_radionullbool_inverse,
        widget=forms.RadioSelect,
        label="Adhérent GAB ?",
    )

    montant_adhesion_min = forms.DecimalField(
        required=False, label="Montant minimum de l'adhésion GAB (en €)"
    )

    montant_adhesion_max = forms.DecimalField(
        required=False, label="Montant maximum de l'adhésion GAB (en €)"
    )

    date_adhesion_anterieure = forms.DateField(
        required=False,
        widget=admin.widgets.AdminDateWidget,
        label="Ayant adhéré après le",
    )

    date_adhesion_posterieure = forms.DateField(
        required=False,
        widget=admin.widgets.AdminDateWidget,
        label="Ayant adhéré avant le",
    )

    date_renouvellement_anterieure = forms.DateField(
        required=False,
        widget=admin.widgets.AdminDateWidget,
        label="Ayant renouvelé son adhésion après le",
    )

    date_renouvellement_posterieure = forms.DateField(
        required=False,
        widget=admin.widgets.AdminDateWidget,
        label="Ayant renouvelé son adhésion avant le",
    )

    def clean(self):
        """
        Vérification des contraintes sur les valeurs de filtrages.
        """
        cdata = super().clean()
        rdv_min = cdata.get('date_rdv_anterieure')
        rdv_max = cdata.get('date_rdv_posterieure')
        if rdv_min is not None and rdv_max is not None and rdv_min > rdv_max:
            raise forms.ValidationError(
                "la date de rendez-vous minimale"
                " est postérieure à la date maximale"
            )
        montant_min = cdata.get('montant_adhesion_min')
        montant_max = cdata.get('montant_adhesion_max')
        if (
            montant_min is not None
            and montant_max is not None
            and montant_min > montant_max
        ):
            raise forms.ValidationError(
                "le montant minimal d'adhésion"
                " est postérieure au montant maximal"
            )
        adhesion_min = cdata.get('date_adhesion_anterieure')
        adhesion_max = cdata.get('date_adhesion_posterieure')
        if (
            adhesion_min is not None
            and adhesion_max is not None
            and adhesion_min > adhesion_max
        ):
            raise forms.ValidationError(
                "la date d'adhésion minimale"
                " est postérieure à la date maximale"
            )
        renouvellement_min = cdata.get('date_renouvellement_anterieure')
        renouvellement_max = cdata.get('date_renouvellement_posterieure')
        if (
            renouvellement_min is not None
            and renouvellement_max is not None
            and renouvellement_min > renouvellement_max
        ):
            raise forms.ValidationError(
                "la date de renouvellement minimale"
                " est postérieure à la date maximale"
            )
        return cdata


class StructureFilterForm(AsDlMixin, forms.Form):
    """
    un filter_set est une liste de triplet: (champs, lookup, valeur_forcee)
    si valeur_forcee vaut None le filtrage est fait simplement:
    queryset.filter(<lookup>=form.cleaned_data[<champs>])

    Ex: ('statutbio', 'statutbio', None)
    avec dans la Form:
        statutbio = forms.ChoiceField(...)

    va filtrer en recuperant le ou les type 'statutbio' indique dans le
    formulaire dans la queryset

    sinon, valeur_forcee est un doublet: (valeur_reconnue, valeur_utilisee)
    si form.cleaned_data[<champs>] == <valeur_reconnue>
    alors queryset.filter(<lookup>=<valeur_utilisee>)

    Ex: ('est_agri', 'statutbio__gt', (True, 1) )
    avec dans la Form:
        est_agri forms.BooleanField(...)

    si la case "est_agri" est cochee, le filtrage sur la queryset sera
    queryset.filter(statutbio__gt=1)
    (sachant qu'ici un statutbio superieur a 1 represente un agriculteur,
    1 pour agriculteur, 2 pour agriculteur bio).


    le exclude_set fonctionne pareil mais avec un exclude a la place de filter
    """

    filter_set = (
        # field name, lookup, (field_val, forced_val)
        #
        ('designation', 'designation__icontains', ''),
        ('commune', 'commune__nom__icontains', ''),
    )
    exclude_set = ()

    # statutbio = forms.ChoiceField(
    #    required=False,# widget=forms.CheckboxSelectMultiple,
    #    label=u"Type personne morale", choices=fch.STATUT_BIO[1:])

    designation = forms.CharField(required=False, label="Nom")

    commune = forms.CharField(required=False, label="Commune")


# EOF
