# -*- coding: utf-8 -*-
from django.contrib.auth import models as am
from django.core.exceptions import ValidationError
from django.test import TestCase

import pytest

from gaby.contacts import models as cm
from gaby.contacts.tests.utils import cfty, mfty
from gaby.references import models as rm

from .factories import StructureFactory


class TestStructureValidation:
    @pytest.mark.parametrize(
        ('siret', 'is_invalid'),
        [('', False), ('invalid', True), ('45383795700043', False)],
    )
    @pytest.mark.django_db
    def test_structure_siret_valid(self, siret, is_invalid):
        structure = StructureFactory.build(siret=siret)
        error_codes = []
        try:
            structure.clean()
        except ValidationError as err:
            error_codes = [e.code for e in err.error_dict.get('siret', [])]
        assert ('siret_invalid' in error_codes) == is_invalid

    @pytest.mark.parametrize(
        ('siret', 'is_dup'),
        [('', False), ('45383795700043', True), ('45383795700068', False)],
    )
    @pytest.mark.django_db
    def test_structure_siret_dup(self, siret, is_dup):
        StructureFactory(siret='45383795700043')
        structure = StructureFactory(siret=siret)
        error_codes = []
        try:
            structure.full_clean()
        except ValidationError as err:
            error_codes = [e.code for e in err.error_dict.get('siret', [])]
        assert ('siret_dup' in error_codes) == is_dup


class ValStructure(TestCase):
    def setUp(self):
        self.usr1 = am.User.objects.create_user('usr1')
        self.fjur = rm.FormeJuridique()
        self.fjur.save()
        self.i_pm = {
            'date_creation': '2014-10-01',
            'date_derniere_modification': '2014-10-02',
            'utilisateur_derniere_modification': self.usr1,
            'f_juridique': self.fjur,
            'designation': "test",
        }
        self.i_p = {
            'nom': 'testin',
            'prenom': 'testons',
            'date_creation': '2014-10-01',
            'date_derniere_modification': '2014-10-02',
            'utilisateur_derniere_modification': self.usr1,
        }

    def test_structure_f_juridique(self):
        """Tout type de structure doit avoir une forme juridique"""
        # Une structure sans forme juridique avec erreur
        structure = mfty(cm.Structure, self.i_pm, statutbio=0)
        structure.f_juridique = None
        self.assertRaises(ValidationError, structure.full_clean)
        # Une structure avec forme juridique sans erreur
        structure.f_juridique = self.fjur
        structure.full_clean()

    def test_structure_liste_contact(self):
        """La méthode liste_contacts_associes doit retourner la liste
        des personnes associées"""
        # Une structure sans personne associée
        structure = cfty(cm.Structure, self.i_pm, statutbio=0)
        liste_contact = structure.liste_contacts_associes()
        self.assertEqual(len(liste_contact), 0)
        # Une structure avec une personne associée
        personne1 = mfty(cm.Personne, self.i_p, structure=structure)
        personne1.nom = "test1"
        personne1.save()
        liste_contact = structure.liste_contacts_associes()
        self.assertEqual(len(liste_contact), 1)
        self.assertEqual(liste_contact[0].nom, "test1")
        # Une structure avec deux personnes associées
        personne2 = mfty(cm.Personne, self.i_p, structure=structure)
        personne2.nom = "test2"
        personne2.save()
        liste_contact = structure.liste_contacts_associes()
        self.assertEqual(len(liste_contact), 2)
        self.assertEqual(liste_contact[1].nom, "test2")

    def test_structure_adresse_electronique(self):
        """La méthode adresse_electronique doit retourner le nom de la
        structure et son adresse electronique, ou rien si aucune
        adresse email n'est renseignée"""
        # Une structure sans adresse electronique
        structure = mfty(cm.Structure, self.i_pm, statutbio=0)
        adresse_elec = structure.adresse_electronique()
        self.assertEqual(adresse_elec, ())
        #  Une structure avec adresse electronique
        structure.courriel = "structure@test.com"
        adresse_elec = structure.adresse_electronique()
        self.assertEqual(adresse_elec, ("test", "structure@test.com"))

    def test_structure_get_courriels(self):
        """La méthode get_courriels doit retourner l'adresse email de
        la structure et de toutes les personne qui y sont associées
        sauf s'ils ne veulent pas recevoir d'e-mail"""
        #  Une structure sans personne associée et sans adresse email
        structure = cfty(cm.Structure, self.i_pm, statutbio=0)
        courriels = structure.get_courriels()
        self.assertEqual(courriels, [])
        # Une structure sans personne associée et avec adresse email
        structure.courriel = "structure@test.com"
        courriels = structure.get_courriels()
        self.assertEqual(courriels, [("test", "structure@test.com")])
        # Une structure avec personnes associées et avec adresse email
        personne1 = mfty(
            cm.Personne, self.i_p, structure=structure, nom="personne1"
        )
        personne1.courriel = "personne1@test.com"
        personne1.pas_de_courriel = False
        personne1.save()
        personne2 = mfty(
            cm.Personne, self.i_p, structure=structure, nom="personne2"
        )
        personne2.courriel = "personne2@test.com"
        personne2.pas_de_courriel = False
        personne2.save()
        courriels = structure.get_courriels()
        self.assertEqual(
            courriels,
            [
                ("test", "structure@test.com"),
                ("testons personne1", "personne1@test.com"),
                ("testons personne2", "personne2@test.com"),
            ],
        )


# EOF
